@@ -1,21 +1,7 @@
 /* Utilities for reading/writing fstab, mtab, etc.
- * Copyright (C) 1995-2000, 2001, 2002, 2003, 2006
- * Free Software Foundation, Inc.
- * This file is part of the GNU C Library.
  *
- * The GNU C Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * SPDX-License-Identifier: LGPL-2.1+
  *
- * The GNU C Library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifndef _GNU_SOURCE
@@ -125,8 +111,10 @@ struct mntent *getmntent_r(FILE *stream, struct mntent *mp, char *buffer, int bu
 		switch (ret) {
 		case 0:
 			mp->mnt_freq = 0;
+			break;
 		case 1:
 			mp->mnt_passno = 0;
+			break;
 		case 2:
 			break;
 		}
